#ifndef __CLF_CAMERA__
#define __CLF_CAMERA__

class cCamera{
public:
	clf_vector3D m_pos;
	clf_vector3D m_look;
	clf_vector3D m_up;

	cCamera(){
		// setup a default viewer
		setup(clf_vector3D(0,0,3),        // position viewer
					clf_vector3D(0,0,0),        // setup the point of interest (where we are looking at)
					clf_vector3D(0,1,0));       // which way is up
	}
	~cCamera(){}
	void setup(clf_vector3D& pos, clf_vector3D& lookat, clf_vector3D& up){
		m_pos = pos;
		m_look = lookat;
		m_up = up;
	}
	void move_to(clf_vector3D& pos){
		m_pos = pos;
	}
	void look_at(clf_vector3D& lookat){
		m_look = lookat;
	}
	void orient(clf_vector3D& up){
		m_up = up;
	}
	void apply(){
		// the following is 
		gluLookAt(m_pos.x, m_pos.y, m_pos.z,     // viewer position
					m_look.x, m_look.y, m_look.z,      // focus point
					m_up.x, m_up.y, m_up.z);           // up direction
	}
};

class clf_stage{
public:
	clf_stage(){
		left = bottom = 0;
		near_plane = 1.f;
		far_plane = 1000.f;
		perspective_angle = 30;
	}
	~clf_stage(){
	}

	void resize(int l, int b, int w, int h){
		left = l;
		bottom = b;
		width = w;
		height = h;
	}

	void set_perspective(float angle, float np, float fp){
		perspective_angle = angle;
		near_plane = np;
		far_plane = fp;
	}

	void viewport(){
		glViewport (left, bottom, (GLsizei)(width), (GLsizei)(height));				// reset the current viewport
	}
	void perspective(){
		float w = (float)width;
		float h = (float)height;
		gluPerspective (perspective_angle, w/h, near_plane, far_plane);	// calculate the aspect ratio of the window
	}

	void prepare(){
		glViewport (left, bottom, (GLsizei)(width), (GLsizei)(height));				// reset the current viewport
		// glMatrixMode (GL_PROJECTION);										// select the projection matrix
		// glLoadIdentity ();													// reset the projection matrix

		float w = (float)width;
		float h = (float)height;

		// the two paths are exactly the same (internally)
		// in the first we calculate the dimensions of the near plane visible area
		// which requires some elementary but sometimes forgotten trigonometry
		// the second is provided by the utility functions in OpenGL
#if 0
		float r = w / h;
		float persp_angle = perspective_angle/2.f;      // half of the perspective angle goes to one side
		float ys = near_plane*(float)tan(persp_angle*PI/180.f);
		float xs = ys*r;
		glFrustum( -xs, xs, -ys, ys, near_plane, far_plane );
#else
		gluPerspective (perspective_angle, w/h, near_plane, far_plane);	// calculate the aspect ratio of the window
#endif
	}

	int left, bottom;
	int width, height;
	float near_plane, far_plane;
	float perspective_angle;
};

#endif // __CLF_CAMERA__
