#ifndef __CLF_GEOMETRY__
#define __CLF_GEOMETRY__

#include <vector>
#include <list>

#define PI 3.141592653589793

class clf_point2D{
public:
    float x;
    float y;
    clf_point2D():x(0.f), y(0.f){
    }
    clf_point2D(float _x, float _y):x(_x), y(_y){
    }
    clf_point2D(int _x, int _y):x((float)_x), y((float)_y){
    }
    ~clf_point2D(){
    }
};

class clf_rect2D{
public:
    float left;
    float top;
    float right;
    float bottom;
    clf_rect2D() : left(0.f),top(0.f),right(0.f),bottom(0.f) {
    }
    clf_rect2D(float l, float t, float r, float b) : left(l),top(t),right(r),bottom(b) {
    }
    clf_rect2D(clf_rect2D& p) : left(p.left),top(p.top),right(p.right),bottom(p.bottom) {
    }
    ~clf_rect2D(){
    }
};

class clf_polygon2D : public std::vector< clf_point2D>{
public:
	clf_rect2D bounds;
	clf_polygon2D(){
	}
    ~clf_polygon2D(){
	}

	void add_point(const clf_point2D& pt){
		if (size() == 0)
		{
			bounds = clf_rect2D(pt.x, pt.y, pt.x, pt.y);
		}
		else
		{
			if (bounds.left > pt.x) bounds.left = pt.x;
			if (bounds.right < pt.x) bounds.right = pt.x;
			if (bounds.top < pt.y) bounds.top = pt.y;
			if (bounds.bottom > pt.y) bounds.bottom = pt.y;
		}
		push_back(clf_point2D(pt.x, pt.y));
	}
};

class clf_triangle2D
{
public:
	clf_triangle2D(){}
	clf_triangle2D(const clf_triangle2D& t){
		verts[0] = t.verts[0];
		verts[1] = t.verts[1];
		verts[2] = t.verts[2];
	}
	~clf_triangle2D(){}

	clf_point2D verts[3];
};

bool intersect(const clf_point2D& p1, const clf_point2D& p2, const clf_point2D& p3, const clf_point2D& p4, clf_point2D* p0);
bool point_in_polygon(clf_polygon2D& poly, clf_point2D pt);
bool polygons_collide(clf_polygon2D& poly1, clf_polygon2D& poly2);
void rotate2D(float cx, float cy, float *x, float *y, float angle);
// check for intersection, but exclude the vertices
bool intersect_no_vertex(const clf_point2D& p1, const clf_point2D& p2, const clf_point2D& p3, const clf_point2D& p4, clf_point2D* p0);
// check if the line segment (p1,p2) is inside a polygon
bool line_in_polygon(clf_polygon2D& poly, clf_point2D& p1, clf_point2D& p2);
bool triangulate(clf_polygon2D& poly, std::vector<clf_triangle2D>& result);

//////////////////////////////////////////////////////////////////////////////////
/////////////// 3D section

struct clf_vector3D{
	clf_vector3D(){
		x = y = z = 0;
	}
	clf_vector3D(float _x, float _y, float _z):x(_x), y(_y), z(_z){
	}
	clf_vector3D(clf_vector3D& v):x(v.x), y(v.y), z(v.z){
	}
	float x,y,z;
	clf_vector3D& operator=(clf_vector3D& v){
		x = v.x;
		y = v.y;
		z = v.z;
		return *this;
	}
};

struct clf_vector4D{
	clf_vector4D(){
		x = y = z = w = 0;
	}
	clf_vector4D(float _x, float _y, float _z, float _w):x(_x), y(_y), z(_z), w(_w){
	}
	clf_vector4D(clf_vector4D& v):x(v.x), y(v.y), z(v.z), w(v.w){
	}
	float x,y,z,w;
	clf_vector4D& operator=(clf_vector4D& v){
		x = v.x;
		y = v.y;
		z = v.z;
		w = v.w;
		return *this;
	}
};



#endif // __CLF_GEOMETRY__
