#include "stdafx.h"
#include "clf_primitives.h"

void drawBox(GLfloat sizex, GLfloat sizey, GLfloat sizez)
{
	static GLfloat n[6][3] =
	{
		{-1.0, 0.0, 0.0},	// left face     *
		{0.0, 1.0, 0.0},  // top face      *
		{1.0, 0.0, 0.0},  // right face    *
		{0.0, -1.0, 0.0}, // bottom face   *
		{0.0, 0.0, 1.0},  // front face    *
		{0.0, 0.0, -1.0}  // back face     *
	};
	static GLint faces[6][4] =
	{
		{3, 0, 1, 2},	// left face     *
		{6, 7, 3, 2}, // top face      *
		{6, 5, 4, 7}, // right face    *
		{4, 5, 1, 0}, // bottom face   *
		{2, 1, 5, 6}, // front face    *
		{7, 4, 0, 3}  // back face     *
	};

	GLfloat v[8][3];
	GLint i;

	v[0][0] = v[1][0] = v[2][0] = v[3][0] = -sizex / 2;
	v[4][0] = v[5][0] = v[6][0] = v[7][0] = sizex / 2;

	v[0][1] = v[1][1] = v[4][1] = v[5][1] = -sizey / 2;
	v[2][1] = v[3][1] = v[6][1] = v[7][1] = sizey / 2;

	v[0][2] = v[3][2] = v[4][2] = v[7][2] = -sizez / 2;
	v[1][2] = v[2][2] = v[5][2] = v[6][2] = sizez / 2;

	for (i = 5; i >= 0; i--) 
	{
		glBegin(GL_QUADS);
		glNormal3fv(&n[i][0]);
		glVertex3fv(&v[faces[i][0]][0]);
		glVertex3fv(&v[faces[i][1]][0]);
		glVertex3fv(&v[faces[i][2]][0]);
		glVertex3fv(&v[faces[i][3]][0]);
		glEnd();
	}
}

#define DIVISIONS 180
#define SSLICES DIVISIONS
#define SSTACKS DIVISIONS
#define CACHE_SIZE  SSLICES+1
#define SIN sin
#define COS cos
#define PI 3.1415926535897f
void drawSphere(GLfloat radius/*, GLint SSLICES, GLint SSTACKS*/)
{
	GLint i,j;
	static GLfloat sinCache1a[CACHE_SIZE];
	static GLfloat cosCache1a[CACHE_SIZE];
	static GLfloat sinCache2a[CACHE_SIZE];
	static GLfloat cosCache2a[CACHE_SIZE];
	static GLfloat sinCache3a[CACHE_SIZE];
	static GLfloat cosCache3a[CACHE_SIZE];
	static GLfloat sinCache1b[CACHE_SIZE];
	static GLfloat cosCache1b[CACHE_SIZE];
	static GLfloat sinCache2b[CACHE_SIZE];
	static GLfloat cosCache2b[CACHE_SIZE];
	static GLfloat sinCache3b[CACHE_SIZE];
	static GLfloat cosCache3b[CACHE_SIZE];
	static bool cache_init = false;
	GLfloat angle;
	GLfloat zLow, zHigh;
	GLfloat sintemp1 = 0.0, sintemp2 = 0.0, sintemp3 = 0.0, sintemp4 = 0.0;
	GLfloat costemp1 = 0.0, costemp2 = 0.0, costemp3 = 0.0, costemp4 = 0.0;
	GLint start, finish;

	//if (!cache_init)
	{
		cache_init = true;

		for (i = 0; i < SSLICES; i++) 
		{
			// angle = /* 2 * PI * */ 360.f*i / SSLICES;
			angle = 2.f * PI * (float)i / (float)SSLICES;
			sinCache1a[i] = SIN(angle);
			cosCache1a[i] = COS(angle);
			sinCache2a[i] = sinCache1a[i];
			cosCache2a[i] = cosCache1a[i];
		}

		for (j = 0; j <= SSTACKS; j++) 
		{
			angle = PI * (float)j / (float)SSTACKS;
			sinCache2b[j] = SIN(angle);
			cosCache2b[j] = COS(angle);
			sinCache1b[j] = radius * SIN(angle);
			cosCache1b[j] = radius * COS(angle);
		}
		/* Make sure it comes to a point */
		sinCache1b[0] = 0;
		sinCache1b[SSTACKS] = 0;

		for (i = 0; i < SSLICES; i++) 
		{
			angle = 2.f * PI * (float)i / (float)SSLICES;
			sinCache3a[i] = SIN(angle);
			cosCache3a[i] = COS(angle);
		}
		for (j = 0; j <= SSTACKS; j++) 
		{
			angle = PI * (float)j / (float)SSTACKS;
			sinCache3b[j] = SIN(angle);
			cosCache3b[j] = COS(angle);
		}

		sinCache1a[SSLICES] = sinCache1a[0];
		cosCache1a[SSLICES] = cosCache1a[0];
		sinCache2a[SSLICES] = sinCache2a[0];
		cosCache2a[SSLICES] = cosCache2a[0];
		sinCache3a[SSLICES] = sinCache3a[0];
		cosCache3a[SSLICES] = cosCache3a[0];
	}

	start = 1;
	finish = SSTACKS - 1;

	/* Low end first (j == 0 iteration) */
	sintemp2 = sinCache1b[1];
	zHigh = cosCache1b[1];
	sintemp3 = sinCache2b[1];
	costemp3 = cosCache2b[1];
	glNormal3f(sinCache2a[0] * sinCache2b[0], cosCache2b[0], cosCache2a[0] * sinCache2b[0]);

	glBegin(GL_TRIANGLE_FAN);
	glVertex3f(0.0, radius, 0.0);
	for (i = SSLICES; i >= 0; i--) 
	{
		glNormal3f(sinCache2a[i] * sintemp3, costemp3, cosCache2a[i] * sintemp3);
		glVertex3f(sintemp2 * sinCache1a[i], zHigh, sintemp2 * cosCache1a[i]);
	}
	glEnd();

	/* High end next (j == SSTACKS-1 iteration) */
	sintemp2 = sinCache1b[SSTACKS-1];
	zHigh = cosCache1b[SSTACKS-1];
	sintemp3 = sinCache2b[SSTACKS-1];
	costemp3 = cosCache2b[SSTACKS-1];
	glNormal3f(sinCache2a[SSTACKS] * sinCache2b[SSTACKS], cosCache2b[SSTACKS], cosCache2a[SSTACKS] * sinCache2b[SSTACKS]);
	glBegin(GL_TRIANGLE_FAN);
	glVertex3f(0.0, -radius, 0.0);
	for (i = 0; i <= SSLICES; i++) 
	{
		glNormal3f(sinCache2a[i] * sintemp3, costemp3, cosCache2a[i] * sintemp3);
		glVertex3f(sintemp2 * sinCache1a[i], zHigh, sintemp2 * cosCache1a[i]);
	}
	glEnd();

		for (j = start; j < finish; j++) 
		{
			zLow = cosCache1b[j];
			zHigh = cosCache1b[j+1];
			sintemp1 = sinCache1b[j];
			sintemp2 = sinCache1b[j+1];
					sintemp3 = sinCache2b[j+1];
					costemp3 = cosCache2b[j+1];
					sintemp4 = sinCache2b[j];
					costemp4 = cosCache2b[j];

			glBegin(GL_QUAD_STRIP);
			for (i = 0; i <= SSLICES; i++) 
			{
					glNormal3f(sinCache2a[i] * sintemp3, costemp3, cosCache2a[i] * sintemp3);
					glVertex3f(sintemp2 * sinCache1a[i], zHigh, sintemp2 * cosCache1a[i]);
					glNormal3f(sinCache2a[i] * sintemp4, costemp4, cosCache2a[i] * sintemp4);
					glVertex3f(sintemp1 * sinCache1a[i], zLow, sintemp1 * cosCache1a[i]);
			}
			glEnd();
		}
}
